@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """
 Writing tests with Hypothesis frees you from the tedium of deciding on and
@@ -53,6 +48,11 @@ generally do their best to write you a useful test.  You can also use
       -e, --except OBJ_NAME      dotted name of exception(s) to ignore
       -h, --help                 Show this message and exit.
 
+.. tip::
+
+    Using a light theme?  Hypothesis respects `NO_COLOR <https://no-color.org/>`__
+    and :envvar:`DJANGO_COLORS=light <django:DJANGO_COLORS>`.
+
 .. note::
 
     The ghostwriter requires :pypi:`black`, but the generated code only
@@ -221,9 +221,9 @@ def _type_from_doc_fragment(token: str) -> Optional[type]:
         return int
     if "numpy" in sys.modules:
         if re.fullmatch(r"[Aa]rray[-_ ]?like", token):
-            return sys.modules["numpy"].ndarray  # type: ignore
+            return sys.modules["numpy"].ndarray
         elif token == "dtype":
-            return sys.modules["numpy"].dtype  # type: ignore
+            return sys.modules["numpy"].dtype
     # Natural-language syntax, e.g. "sequence of integers"
     coll_match = re.fullmatch(r"(\w+) of (\w+)", token)
     if coll_match is not None:
@@ -252,9 +252,9 @@ def _strategy_for(
     # - `b (list, tuple, or None): ...`
     # - `c : {"foo", "bar", or None}`
     for pattern in (
-        fr"^\s*\:type\s+{param.name}\:\s+(.+)",  # RST-style
-        fr"^\s*{param.name} \((.+)\):",  # Google-style
-        fr"^\s*{param.name} \: (.+)",  # Numpy-style
+        rf"^\s*\:type\s+{param.name}\:\s+(.+)",  # RST-style
+        rf"^\s*{param.name} \((.+)\):",  # Google-style
+        rf"^\s*{param.name} \: (.+)",  # Numpy-style
     ):
         match = re.search(pattern, docstring, flags=re.MULTILINE)
         if match is None:
@@ -333,8 +333,7 @@ def _get_params(func: Callable) -> Dict[str, inspect.Parameter]:
             and isinstance(func.__doc__, str)
         ):
             # inspect.signature doesn't work on all builtin functions or methods.
-            # In such cases, including the operator module on Python 3.6, we can try
-            # to reconstruct simple signatures from the docstring.
+            # In such cases, we can try to reconstruct simple signatures from the docstring.
             match = re.match(rf"^{func.__name__}\((.+?)\)", func.__doc__)
             if match is None:
                 raise
@@ -461,9 +460,6 @@ def _imports_for_object(obj):
         name = _get_qualname(obj).split(".")[0]
         return {(_get_module(obj), name)}
     except Exception:
-        with contextlib.suppress(AttributeError):
-            if obj.__module__ == "typing":  # only on CPython 3.6
-                return {("typing", getattr(obj, "__name__", obj.name))}
         return set()
 
 
@@ -779,7 +775,7 @@ def magic(
             functions.add(thing)
         elif isinstance(thing, types.ModuleType):
             if hasattr(thing, "__all__"):
-                funcs = [getattr(thing, name, None) for name in thing.__all__]  # type: ignore
+                funcs = [getattr(thing, name, None) for name in thing.__all__]
             else:
                 pkg = thing.__package__
                 funcs = [
