@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """This module provides support for a stateful style of testing, where tests
 attempt to find a sequence of operations that cause a breakage rather than just
@@ -25,7 +20,6 @@ import inspect
 from copy import copy
 from functools import lru_cache
 from io import StringIO
-from operator import attrgetter
 from typing import (
     Any,
     Callable,
@@ -53,7 +47,12 @@ from hypothesis.core import TestFunc, given
 from hypothesis.errors import InvalidArgument, InvalidDefinition
 from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.healthcheck import fail_health_check
-from hypothesis.internal.reflection import function_digest, nicerepr, proxies
+from hypothesis.internal.reflection import (
+    function_digest,
+    get_pretty_function_description,
+    nicerepr,
+    proxies,
+)
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import current_verbosity, report
 from hypothesis.strategies._internal.featureflags import FeatureStrategy
@@ -334,16 +333,16 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
 
     def _print_step(self, rule, data, result):
         self.step_count = getattr(self, "step_count", 0) + 1
-        # If the step has target bundles, and the result is a MultipleResults
-        # then we want to assign to multiple variables.
-        if isinstance(result, MultipleResults):
-            n_output_vars = len(result.values)
-        else:
-            n_output_vars = 1
-        if rule.targets and n_output_vars >= 1:
-            output_assignment = ", ".join(self._last_names(n_output_vars)) + " = "
-        else:
-            output_assignment = ""
+        output_assignment = ""
+        if rule.targets:
+            if isinstance(result, MultipleResults):
+                if len(result.values) == 1:
+                    output_assignment = f"({self._last_names(1)[0]},) = "
+                elif result.values:
+                    output_names = self._last_names(len(result.values))
+                    output_assignment = ", ".join(output_names) + " = "
+            else:
+                output_assignment = self._last_names(1)[0] + " = "
         report(
             "{}state.{}({})".format(
                 output_assignment,
@@ -367,6 +366,11 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
                 continue
             if not all(precond(self) for precond in invar.preconditions):
                 continue
+            if (
+                current_build_context().is_final
+                or settings.verbosity >= Verbosity.debug
+            ):
+                report(f"state.{invar.function.__name__}()")
             result = invar.function(self)
             if result is not None:
                 fail_health_check(
@@ -405,7 +409,7 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
 @attr.s()
 class Rule:
     targets = attr.ib()
-    function = attr.ib(repr=attrgetter("__qualname__"))
+    function = attr.ib(repr=get_pretty_function_description)
     arguments = attr.ib()
     preconditions = attr.ib()
     bundles = attr.ib(init=False)
@@ -823,7 +827,7 @@ def precondition(precond: Callable[[Any], bool]) -> Callable[[TestFunc], TestFun
 
 @attr.s()
 class Invariant:
-    function = attr.ib()
+    function = attr.ib(repr=get_pretty_function_description)
     preconditions = attr.ib()
     check_during_init = attr.ib()
 
