@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import enum
 import math
@@ -24,7 +19,7 @@ import typing
 from decimal import Context, Decimal, localcontext
 from fractions import Fraction
 from functools import reduce
-from inspect import Parameter, Signature, getfullargspec, isabstract, isclass, signature
+from inspect import Parameter, Signature, isabstract, isclass, signature
 from types import FunctionType
 from typing import (
     Any,
@@ -53,13 +48,7 @@ from hypothesis.control import cleanup, note
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.charmap import as_general_categories
-from hypothesis.internal.compat import (
-    ceil,
-    floor,
-    get_type_hints,
-    is_typed_named_tuple,
-    typing_root_type,
-)
+from hypothesis.internal.compat import ceil, floor, get_type_hints, is_typed_named_tuple
 from hypothesis.internal.conjecture.utils import (
     calc_label_from_cls,
     check_sample,
@@ -67,7 +56,7 @@ from hypothesis.internal.conjecture.utils import (
 )
 from hypothesis.internal.entropy import get_seeder_and_restorer
 from hypothesis.internal.reflection import (
-    define_function_signature,
+    define_function_signature_from_signature,
     get_pretty_function_description,
     nicerepr,
     required_args,
@@ -751,7 +740,7 @@ class RandomSeeder:
 
 class RandomModule(SearchStrategy):
     def do_draw(self, data):
-        seed = data.draw(integers(0, 2 ** 32 - 1))
+        seed = data.draw(integers(0, 2**32 - 1))
         seed_all, restore_all = get_seeder_and_restorer(seed)
         seed_all()
         cleanup(restore_all)
@@ -809,6 +798,13 @@ class BuildsStrategy(SearchStrategy):
                         f"error - try using from_type({self.target!r}) instead "
                         f"of builds({self.target!r})"
                     ) from err
+            if getattr(self.target, "__no_type_check__", None) is True:
+                # Note: could use PEP-678 __notes__ here.  Migrate over once we're
+                # using an `exceptiongroup` backport with support for that.
+                raise TypeError(
+                    "This might be because the @no_type_check decorator prevented "
+                    "Hypothesis from inferring a strategy for some required arguments."
+                ) from err
             raise
 
     def validate(self):
@@ -854,7 +850,7 @@ def builds(
     the callable.
     """
     if not callable_and_args:
-        raise InvalidArgument(
+        raise InvalidArgument(  # pragma: no cover
             "builds() must be passed a callable as the first positional "
             "argument, but no positional arguments were given."
         )
@@ -886,8 +882,21 @@ def builds(
             raise InvalidArgument(
                 f"passed infer for {badargs}, but there is no type annotation"
             )
-        for kw in set(hints) & (required | to_infer):
-            kwargs[kw] = from_type(hints[kw])
+        infer_for = {k: v for k, v in hints.items() if k in (required | to_infer)}
+        if infer_for:
+            from hypothesis.strategies._internal.types import _global_type_lookup
+
+            for kw, t in infer_for.items():
+                if (
+                    getattr(t, "__module__", None) in ("builtins", "typing")
+                    or t in _global_type_lookup
+                ):
+                    kwargs[kw] = from_type(t)
+                else:
+                    # We defer resolution of these type annotations so that the obvious
+                    # approach to registering recursive types just works.  See
+                    # https://github.com/HypothesisWorks/hypothesis/issues/3026
+                    kwargs[kw] = deferred(lambda t=t: from_type(t))  # type: ignore
     return BuildsStrategy(target, args, kwargs)
 
 
@@ -896,17 +905,21 @@ if sys.version_info[:2] >= (3, 8):  # pragma: no branch
     # matches the semantics of the function.  Great for documentation!
     sig = signature(builds)
     args, kwargs = sig.parameters.values()
-    builds.__signature__ = sig.replace(
-        parameters=[
-            Parameter(
-                name="target",
-                kind=Parameter.POSITIONAL_ONLY,
-                annotation=Callable[..., Ex],
-            ),
-            args.replace(name="args", annotation=SearchStrategy[Any]),
-            kwargs,
-        ]
-    )
+    builds = define_function_signature_from_signature(
+        name=builds.__name__,
+        docstring=builds.__doc__,
+        signature=sig.replace(
+            parameters=[
+                Parameter(
+                    name="target",
+                    kind=Parameter.POSITIONAL_ONLY,
+                    annotation=Callable[..., Ex],
+                ),
+                args.replace(name="args", annotation=SearchStrategy[Any]),
+                kwargs,
+            ]
+        ),
+    )(builds)
 
 
 @cacheable
@@ -988,16 +1001,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
         # of it.  We do this in three places, hence the helper function
         if not isinstance(strat_or_callable, SearchStrategy):
             assert callable(strat_or_callable)  # Validated in register_type_strategy
-            try:
-                # On Python 3.6, typing.Hashable is just an alias for abc.Hashable,
-                # and the resolver function for Type throws an AttributeError because
-                # Hashable has no __args__.  We discard such errors when attempting
-                # to resolve subclasses, because the function was passed a weird arg.
-                strategy = strat_or_callable(thing)
-            except Exception:  # pragma: no cover
-                if not final:
-                    return nothing()
-                raise
+            strategy = strat_or_callable(thing)
         else:
             strategy = strat_or_callable
         if not isinstance(strategy, SearchStrategy):
@@ -1016,27 +1020,11 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             if thing in types._global_type_lookup:
                 return as_strategy(types._global_type_lookup[thing], thing)
             return from_type(thing.__supertype__)
-        # Under Python 3.6, Unions are not instances of `type` - but we
-        # still want to resolve them!
+        # Unions are not instances of `type` - but we still want to resolve them!
         if getattr(thing, "__origin__", None) is typing.Union:
             args = sorted(thing.__args__, key=types.type_sorting_key)
             return one_of([from_type(t) for t in args])
     if not types.is_a_type(thing):
-        # The implementation of typing_extensions.Literal under Python 3.6 is
-        # *very strange*.  Notably, `type(Literal[x]) != Literal` so we have to
-        # use the first form directly, and because it uses __values__ instead of
-        # __args__ we inline the relevant logic here until the end of life date.
-        if types.is_typing_literal(thing):  # pragma: no cover
-            assert sys.version_info[:2] == (3, 6)
-            args_dfs_stack = list(thing.__values__)  # type: ignore
-            literals = []
-            while args_dfs_stack:
-                arg = args_dfs_stack.pop()
-                if types.is_typing_literal(arg):
-                    args_dfs_stack.extend(reversed(arg.__values__))
-                else:
-                    literals.append(arg)
-            return sampled_from(literals)
         if isinstance(thing, str):
             # See https://github.com/HypothesisWorks/hypothesis/issues/3016
             raise InvalidArgument(
@@ -1046,8 +1034,15 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
                 "strings."
             )
         raise InvalidArgument(f"thing={thing!r} must be a type")  # pragma: no cover
+    if thing in types.NON_RUNTIME_TYPES:
+        # Some code like `st.from_type(TypeAlias)` does not make sense.
+        # Because there are types in python that do not exist in runtime.
+        raise InvalidArgument(
+            f"Could not resolve {thing!r} to a strategy, "
+            f"because there is no such thing as a runtime instance of {thing!r}"
+        )
     # Now that we know `thing` is a type, the first step is to check for an
-    # explicitly registered strategy.  This is the best (and hopefully most
+    # explicitly registered strategy. This is the best (and hopefully most
     # common) way to resolve a type to a strategy.  Note that the value in the
     # lookup may be a strategy or a function from type -> strategy; and we
     # convert empty results into an explicit error.
@@ -1084,7 +1079,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
     # We'll start by checking if thing is from from the typing module,
     # because there are several special cases that don't play well with
     # subclass and instance checks.
-    if isinstance(thing, typing_root_type) or (
+    if isinstance(thing, types.typing_root_type) or (
         sys.version_info[:2] >= (3, 9)
         and isinstance(getattr(thing, "__origin__", None), type)
         and getattr(thing, "__args__", None)
@@ -1479,29 +1474,32 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
     else:
         special_method = None
 
-    argspec = getfullargspec(f)
+    sig = signature(f)
+    params = tuple(sig.parameters.values())
 
-    if argspec.defaults is not None and len(argspec.defaults) == len(argspec.args):
-        raise InvalidArgument("A default value for initial argument will never be used")
-    if len(argspec.args) == 0 and not argspec.varargs:
+    if not (params and "POSITIONAL" in params[0].kind.name):
         raise InvalidArgument(
             "Functions wrapped with composite must take at least one "
             "positional argument."
         )
-
-    annots = {
-        k: v
-        for k, v in argspec.annotations.items()
-        if k in (argspec.args + argspec.kwonlyargs + ["return"])
-    }
-    new_argspec = argspec._replace(args=argspec.args[1:], annotations=annots)
+    if params[0].default is not sig.empty:
+        raise InvalidArgument("A default value for initial argument will never be used")
+    if params[0].kind.name != "VAR_POSITIONAL":
+        params = params[1:]
+    newsig = sig.replace(
+        parameters=params,
+        return_annotation=SearchStrategy
+        if sig.return_annotation is sig.empty
+        else SearchStrategy[sig.return_annotation],  # type: ignore
+    )
 
     @defines_strategy()
-    @define_function_signature(f.__name__, f.__doc__, new_argspec)
+    @define_function_signature_from_signature(f.__name__, f.__doc__, newsig)
     def accept(*args, **kwargs):
         return CompositeStrategy(f, args, kwargs)
 
     accept.__module__ = f.__module__
+    accept.__signature__ = newsig
     if special_method is not None:
         return special_method(accept)
     return accept
@@ -1774,6 +1772,11 @@ def register_type_strategy(
 
     if not types.is_a_type(custom_type):
         raise InvalidArgument(f"custom_type={custom_type!r} must be a type")
+    if custom_type in types.NON_RUNTIME_TYPES:
+        raise InvalidArgument(
+            f"custom_type={custom_type!r} is not allowed to be registered, "
+            f"because there is no such thing as a runtime instance of {custom_type!r}"
+        )
     elif not (isinstance(strategy, SearchStrategy) or callable(strategy)):
         raise InvalidArgument(
             "strategy=%r must be a SearchStrategy, or a function that takes "
@@ -1852,19 +1855,20 @@ def emails() -> SearchStrategy[str]:
 def functions(
     *,
     like: Callable[..., Any] = lambda: None,
-    returns: Optional[SearchStrategy[Any]] = None,
+    returns: Union[SearchStrategy[Any], InferType] = infer,
     pure: bool = False,
 ) -> SearchStrategy[Callable[..., Any]]:
     # The proper type signature of `functions()` would have T instead of Any, but mypy
     # disallows default args for generics: https://github.com/python/mypy/issues/3737
-    """functions(*, like=lambda: None, returns=none(), pure=False)
+    """functions(*, like=lambda: None, returns=infer, pure=False)
 
     A strategy for functions, which can be used in callbacks.
 
     The generated functions will mimic the interface of ``like``, which must
     be a callable (including a class, method, or function).  The return value
     for the function is drawn from the ``returns`` argument, which must be a
-    strategy.
+    strategy.  If ``returns`` is not passed, we attempt to infer a strategy
+    from the return-type annotation if present, falling back to :func:`~none`.
 
     If ``pure=True``, all arguments passed to the generated function must be
     hashable, and if passed identical arguments the original return value will
@@ -1883,7 +1887,9 @@ def functions(
             f"but got non-callable like={nicerepr(like)!r}"
         )
 
-    if returns is None:
+    if returns is None or returns is infer:
+        # Passing `None` has never been *documented* as working, but it still
+        # did from May 2020 to Jan 2022 so we'll avoid breaking it without cause.
         hints = get_type_hints(like)
         returns = from_type(hints.get("return", type(None)))
 
